# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import model_utils.fields
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('surveys', '0004_auto_20161223_1129'),
    ]

    operations = [
        migrations.AddField(
            model_name='rating',
            name='created',
            field=model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False),
        ),
        migrations.AddField(
            model_name='rating',
            name='modified',
            field=model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False),
        ),
        migrations.AddField(
            model_name='ratingtype',
            name='created',
            field=model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False),
        ),
        migrations.AddField(
            model_name='ratingtype',
            name='modified',
            field=model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False),
        ),
        migrations.AddField(
            model_name='review',
            name='modified',
            field=model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False),
        ),
    ]



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import, division

import logging

from compat.models import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db import models
from django.utils import six
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from model_utils.models import TimeStampedModel

from .constants import RATING_TYPES_CHOICES
from .managers import ReviewManager, StarRatedReviewManager
from .settings import surveys_settings
from .signals import post_rating

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class RatingType(TimeStampedModel, models.Model):
    name = models.CharField(choices=RATING_TYPES_CHOICES, max_length=30, unique=True)
    min_value = models.IntegerField()
    max_value = models.IntegerField()

    def __str__(self):
        return six.text_type(self.get_name_display())


@python_2_unicode_compatible
class Rating(TimeStampedModel, models.Model):
    value = models.IntegerField(null=True, blank=True)
    type = models.ForeignKey(RatingType, related_name='values', on_delete=models.CASCADE)

    class Meta:
        ordering = ['value']

    def __str__(self):
        plural = 's' if self.value != 1 else ''
        return six.text_type(
            "{0} {1}{2}".format(self.value, self.type, plural).lower()
        )

    def clean(self):
        if self.value is not None:
            if self.value < self.type.min_value or self.value > self.type.max_value:
                msg = _("Invalid Value: should be between {0} and {1}").format(
                    self.type.min_value,
                    self.type.max_value,
                )
                raise ValidationError({'value': msg})

    def save(self, *args, **kwargs):
        self.full_clean()
        super(Rating, self).save(*args, **kwargs)


@python_2_unicode_compatible
class Review(TimeStampedModel, models.Model):
    object_id = models.CharField(max_length=50, db_index=True)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    content_object = GenericForeignKey()
    rating = models.ForeignKey(Rating, related_name='reviews')
    user = models.ForeignKey(surveys_settings.REVIEWER_MODEL, on_delete=models.CASCADE)
    comment = models.TextField(blank=True)
    would_recommend = models.NullBooleanField(default=None)
    rating_value = None

    objects = ReviewManager()

    class Meta:
        unique_together = [
            ("object_id", "content_type", "user"),
        ]

    def __str__(self):
        return six.text_type("{0} ({1})".format(self.name, self.rating))

    @property
    def name(self):
        """
        Returns the stored user name.
        """
        if self.user is not None:
            return self.user.get_full_name()

    @property
    def is_negative(self):
        return self.rating.value and (self.rating.value < (self.rating.type.max_value / 2))

    def save(self, *args, **kwargs):
        super(Review, self).save(*args, **kwargs)
        rated_obj = self.content_object
        signal_kwargs = {
            'sender': rated_obj.__class__,
            'instance': rated_obj,
            'value': self.rating.value,
            'is_negative': self.is_negative,
        }
        logger.debug("Firing signal with: %s", signal_kwargs)
        post_rating.send(**signal_kwargs)


class StarRatedReview(Review):
    objects = StarRatedReviewManager()

    class Meta:
        proxy = True



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import

from django.contrib.contenttypes.models import ContentType
from django.db import models

from .constants import STAR_RATING_TYPE


class ReviewManager(models.Manager):
    def for_model(self, klass):
        return self.get_queryset().filter(
            content_type=ContentType.objects.get_for_model(klass))

    def for_instance(self, instance):
        return self.get_queryset().filter(
            content_type=ContentType.objects.get_for_model(instance),
            object_id=instance.id
        )


class StarRatedReviewManager(ReviewManager):
    def get_queryset(self):
        return super(StarRatedReviewManager, self).get_queryset().filter(rating__type__name=STAR_RATING_TYPE)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('surveys', '0002_auto_20161221_1120'),
    ]

    operations = [
        migrations.AlterField(
            model_name='review',
            name='timestamp',
            field=model_utils.fields.AutoCreatedField(verbose_name='created', editable=False, default=django.utils.timezone.now),
        ),
    ]



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import

import logging

from rest_framework import serializers

from .constants import STAR_RATING_TYPE
from .models import Rating, RatingType, StarRatedReview

logger = logging.getLogger(__name__)


class StarRatedReviewSerializer(serializers.ModelSerializer):
    rating_value = serializers.IntegerField(required=False)

    class Meta:
        model = StarRatedReview
        fields = (
            'object_id',
            'content_type',
            'rating_value',
            'comment',
            'would_recommend',
            'user',
        )

    def _get_rating_type(self):
        rating_type, created = RatingType.objects.get_or_create(
            name=STAR_RATING_TYPE, defaults={'min_value': 0, 'max_value': 5})
        logger.info("%s rating type %s", created and "Created new" or "Retrieved existing", rating_type)
        return rating_type

    def _get_rating_obj(self, value):
        rating_type = self._get_rating_type()
        rating, created = Rating.objects.get_or_create(type=rating_type, value=value)
        logger.info("%s rating %s", created and "Created new" or "Retrieved existing", rating)
        return rating

    def to_representation(self, instance):
        data = super(StarRatedReviewSerializer, self).to_representation(instance)
        data['rating_value'] = instance.rating.value
        return data

    def create(self, validated_data):
        logger.info("Calling StarRatedReviewSerializer.create()")
        value = validated_data.pop('rating_value', None)
        validated_data['rating'] = self._get_rating_obj(value)
        return super(StarRatedReviewSerializer, self).create(validated_data)



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import

from django.utils.translation import ugettext_lazy as _

STAR_RATING_TYPE = 'star-rating'

RATING_TYPES_CHOICES = (
    (STAR_RATING_TYPE, _("Star")),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from django.contrib import admin

from .constants import STAR_RATING_TYPE
from .models import RatingType, Rating, Review
from .settings import surveys_settings


@admin.register(RatingType)
class RatingTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'min_value', 'max_value')
    search_fields = ('name',)


@admin.register(Rating)
class RatingAdmin(admin.ModelAdmin):
    list_display = ('value', 'type')
    list_filter = ('type',)


class BadRatingFilter(admin.SimpleListFilter):
    title = "Bad Reviews"
    parameter_name = 'bad_reviews'

    def lookups(self, request, model_admin):
        return [
            ('y', "Below 3 stars"),
        ]

    def queryset(self, request, queryset):
        value = self.value()
        if value == 'y':
            return queryset.filter(rating__value__lt=3)
        return queryset


@admin.register(Review)
class ReviewAdmin(admin.ModelAdmin):
    list_display = (
        'review_for',
        'item_type',
        'rating_value',
        'user',
        'created',
        'comment',
        'would_recommend',
    )
    list_filter = (
        BadRatingFilter,
        ('content_type', admin.RelatedOnlyFieldListFilter),
        ('rating', admin.RelatedOnlyFieldListFilter),
        'created',
    )
    search_fields = ('user__first_name', 'user__last_name', 'comment')

    @property
    def media(self):
        media = super(ReviewAdmin, self).media
        fa_css = surveys_settings.FONT_AWESOME_CSS
        media.add_css({'all': (fa_css,)} if fa_css else {})
        return media

    def item_type(self, obj):
        return "{0}".format(obj.content_type).capitalize()

    def review_for(self, obj):
        return "{0}".format(obj.content_object)

    def rating_value(self, obj):
        if obj.rating.type.name == STAR_RATING_TYPE:
            try:
                val = "".join(['<i class="fa fa-star"></i>' for _ in range(obj.rating.value)])
                return """<div style="min-width: {0}px;">{1}</div>""".format(obj.rating.value * 12, val)
            except TypeError:
                # None rating
                return "None"
        return obj.rating

    rating_value.allow_tags = True
    rating_value.short_description = "Rating"
    rating_value.admin_order_field = 'rating__value'



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import

from django import template

register = template.Library()


@register.inclusion_tag('surveys/survey-star-o.svg')
def star_svg(width=50, color=None):
    return {
        'width': width,
        'color': color,
    }
